#include "ImpressionistModel.h"
#include "Brush.h"

////////////////////////////////////////////////////////////////////////
// Class ImpressionistModel Implementation
//
// The Model class corresponds to to the document in the MFC 
// document/view architecture.
//
// This class maintains the size and type of the brush, the origial
// image and the painted image (the canvas).
// 
// This is a good place to add stuffs that's closely related to the
// image, such as finding the gradient and/or the edges.
////////////////////////////////////////////////////////////////////////

ImpressionistModel::ImpressionistModel() 
{
	m_nImageWidth = m_nImageHeight = 0;
	m_pbyBitmap = NULL;

	m_iBrushType = BRUSH_POINT;
	m_iBrushSize = 10;

	m_pbyCanvas = NULL;
}

ImpressionistModel::~ImpressionistModel()
{
	if (m_pbyBitmap) 
		delete[] m_pbyBitmap;
}

bool ImpressionistModel::LoadImage(const char *szFileName) 
{
	unsigned char *pbyData;
	int iWidth, iHeight;

	if ((pbyData = ::ReadBMP(szFileName, &iWidth, &iHeight)) == NULL) {
		return false;
	}

	// release old storage
	if (m_pbyBitmap)
		delete[] m_pbyBitmap;

	m_nImageWidth = iWidth;
	m_nImageHeight = iHeight;
	m_pbyBitmap = pbyData;

	return true;
}

bool ImpressionistModel::SaveImage(const char *szFileName) const
{
	return ::WriteBMP(szFileName, m_nImageWidth, m_nImageHeight, m_pbyCanvas);
}

const GLubyte* ImpressionistModel::GetPixel(unsigned char* pbyImage, int x, int y) const
{
	if (x < 0) 
		x = 0;
	else if (x >= m_nImageWidth) 
		x = m_nImageWidth - 1;

	if (y < 0) 
		y = 0;
	else if (y >= m_nImageHeight) 
		y = m_nImageHeight - 1;

	return (GLubyte*)(pbyImage + 3 * (y * m_nImageWidth + x));
}

void ImpressionistModel::ClearCanvas()
{
	if (m_pbyCanvas) {
		memset(m_pbyCanvas, 255, m_nImageWidth * m_nImageHeight * 3);
	}
}

