#ifndef IMPRESSIONIST_H_INCLUDED
#define IMPRESSIONIST_H_INCLUDED

#include <FL/Fl.h>
#include <FL/gl.h>
#include <GL/glu.h>

#include "Bitmap.h"
#include "Brush.h"

class ImpressionistModel 
{
public:
	ImpressionistModel();
	virtual ~ImpressionistModel();

	bool LoadImage(const char *szFileName);
	bool SaveImage(const char *szFileName) const;

	int ImageWidth() const { return m_nImageWidth; }
	int ImageHeight() const { return m_nImageHeight; }

	// Returns the current brush type
	int BrushType() const { return m_iBrushType; }
	// Change the current brush type
	void BrushType(int iBrushType) { m_iBrushType = iBrushType; }
	// Returns the current brush size
	int BrushSize() const { return m_iBrushSize; }
	// Change the current brush size
	void BrushSize(int iBrushSize) { m_iBrushSize = iBrushSize; }

	bool HasBitmap() const { return (m_pbyBitmap != NULL); }
	const unsigned char* Bitmap() const { return m_pbyBitmap; }
	// Set the canvas buffer. Note that ImpressionistModel is not responsible
	// for reallocation or deallocation of the canvas buffer.
	void Canvas(unsigned char* pbyCanvas) { m_pbyCanvas = pbyCanvas; }
	void ClearCanvas();

	// get the pixel of the original image at the specific point
	const GLubyte* GetOriginalPixel(int x, int y) const { return GetPixel(m_pbyBitmap, x, y); }
	const GLubyte* GetOriginalPixel(const Point& p) const { return GetPixel(m_pbyBitmap, p.x, p.y); }

protected:
	int m_iBrushType;
	int m_iBrushSize;

	int m_nImageWidth, m_nImageHeight;
	unsigned char* m_pbyBitmap;
	// this buffer will be passed in by the paint view window for efficiency
	unsigned char* m_pbyCanvas; 

	const GLubyte* GetPixel(unsigned char* pbyImage, int x, int y) const;
};

#endif // IMPRESSIONIST_H_INCLUDED
