#include "ImpressionistModel.h"
#include "OriginalView.h"

#ifndef min
#define min(a, b) ((a) < (b) ? (a) : (b))
#endif

////////////////////////////////////////////////////////////////////////
// OriginalView Class Implementation
//
// This window is resposible for displaying the original image. You'll
// probably only need to make slight change to this class, if any.
////////////////////////////////////////////////////////////////////////

OriginalView::OriginalView(int x, int y, int w, int h, const char *l) : 
	Fl_Gl_Window(x,y,w,h,l)
{
}

void OriginalView::draw()
{
	if(!valid()) {
		glClearColor(1.0f, 1.0f, 1.0f, 1.0); // white

		// We're only using 2D, so turn off depth test
		glDisable(GL_DEPTH_TEST);

		ortho();
	}

	glClear(GL_COLOR_BUFFER_BIT);

	if (m_pmModel->HasBitmap()) {

		// note that both OpenGL pixel storage and the Windows BMP format
		// store pixels left-to-right, BOTTOM-to-TOP!!  thus all the fiddling
		// around with startrow.

		int drawWidth, drawHeight;
		const GLvoid* pBitStart;

		// we are not using a scrollable window, so ignore it
		Point scrollpos; // = GetScrollPosition();
		scrollpos.x = scrollpos.y = 0;

		drawWidth = min(w(), m_pmModel->ImageWidth());
		drawHeight = min(h(), m_pmModel->ImageHeight());

		int startrow = m_pmModel->ImageHeight() - (scrollpos.y + drawHeight);
		if (startrow < 0) 
			startrow = 0;

		pBitStart = m_pmModel->Bitmap() + 3 * ((m_pmModel->ImageWidth() * startrow) + scrollpos.x);

		glRasterPos2i(0, h() - drawHeight);
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		glPixelStorei(GL_UNPACK_ROW_LENGTH, m_pmModel->ImageWidth());
		glDrawBuffer(GL_BACK);

		glDrawPixels(drawWidth, drawHeight, GL_RGB, GL_UNSIGNED_BYTE, pBitStart);
	}
			
	glFlush();
}

void OriginalView::resize_window(int width, int height)
{
	resize(x(), y(), width, height);
}
